import os
from click import *
from .util import extract_code_from_directory


@group()
def code() -> None:
    pass


@command(help="collect code")
@option("--dirs", help="list of directories", required=True, multiple=True, type=str)
@option(
    "--langs",
    help="list of languages",
    multiple=True,
    type=str,
)
@option("--output", help="output file", required=True)
def collect(dirs, langs, output) -> None:
    print(f"dirs: {dirs}, langs: {langs}, output: {output}")
    extract_code_from_directory(dirs, langs, output)


code.add_command(collect)



import os
from click import *


@group()
def fs() -> None:
    pass


@command(help="creating a file")
@argument("name", required=True)
@argument("content", required=False)
def create(name: str, content: str) -> None:
    with open(name, "w", encoding="utf-8") as f:
        if content:
            f.write(content)


@command(help="cat a file")
@argument("name", required=True)
def cat(name: str) -> None:
    if not os.path.exists(name):
        print(f"file not found: {name}")
        return
    with open(name, "r", encoding="utf-8") as f:
        print(f.read())


@command(help="delete a file")
@argument("name", required=True)
def rm(name: str) -> None:
    os.remove(name)


@command(help="rename a file")
@argument("name", required=True)
@argument("new_name", required=True)
def mv(name: str, new_name: str) -> None:
    os.rename(name, new_name)


@command(help="copy a file")
@argument("name", required=True)
@argument("new_name", required=True)
def cp(name: str, new_name: str) -> None:
    with open(name, "r", encoding="utf-8") as f:
        with open(new_name, "w", encoding="utf-8") as nf:
            nf.write(f.read())


# list files
@command(help="list files")
@option("-a", "--show-all", is_flag=True, help="show hidden files")
@option("-l", "--show-list", is_flag=True, help="show files in a list")
def ls(show_all, show_list) -> None:
    if not show_list:
        print("  ".join(os.listdir()))
    else:
        for file in os.listdir():
            print(file)


fs.add_command(create)
fs.add_command(rm)
fs.add_command(cat)
fs.add_command(mv)
fs.add_command(cp)
fs.add_command(ls)



import os
import re
from typing import Set, Tuple

# 为不同的语言定义正注释行检测正则表达式
REGS_FOR_LANGS = {
    "python": r"^\s*#",
    "py": r"^\s*#",
    "javascript": r"^\s*//",
    "js": r"^\s*//",
    "jsx": r"^\s*//",
    "typescript": r"^\s*//",
    "ts": r"^\s*//",
    "tsx": r"^\s*//",
    "java": r"^\s*//",
    "c": r"^\s*//",
    "cpp": r"^\s*//",
    "go": r"^\s*//",
    "golang": r"^\s*//",
    "rust": r"^\s*//",
    "rs": r"^\s*//",
    "shell": r"^\s*#",
    "sh": r"^\s*#",
    "ruby": r"^\s*#",
    "perl": r"^\s*#",
    "php": r"^\s*//",
    "swift": r"^\s*//",
    "kotlin": r"^\s*//",
    "scala": r"^\s*//",
    "groovy": r"^\s*//",
    "lua": r"^\s*--",
    "r": r"^\s*#",
    "matlab": r"^\s*%",
    "julia": r"^\s*#",
    "haskell": r"^\s*--",
    "erlang": r"^\s*%",
    "elixir": r"^\s*#",
    "clojure": r"^\s*;",
}

# 空白行正则表达式
REG_BLANK_LINE = r"^\s*$"

# 扩展名到语言名映射
EXT_LANG_MAPPING = {
    ".py": "python",
    ".js": "javascript",
    ".ts": "typescript",
    ".java": "java",
    ".c": "c",
    ".cpp": "cpp",
    ".go": "go",
    ".rs": "rust",
    ".sh": "shell",
    ".rb": "ruby",
    ".pl": "perl",
    ".php": "php",
    ".swift": "swift",
    ".kt": "kotlin",
    ".scala": "scala",
    ".groovy": "groovy",
    ".lua": "lua",
    ".r": "r",
    ".m": "matlab",
    ".jl": "julia",
    ".hs": "haskell",
    ".erl": "erlang",
    ".ex": "elixir",
    ".clj": "clojure",
}


def get_supported_lang(lang_ext: str) -> str:
    """
    获取支持的语言名称

    :param lang_ext: 语言扩展名, 以 "." 开头
    :type lang_ext: str
    :return: 对应的语言名称
    :rtype: str
    """
    return lang_ext[1:] if lang_ext in EXT_LANG_MAPPING else ""


def is_comment_or_blank(lang: str, line: str):
    comment_reg = REGS_FOR_LANGS.get(lang, r"^\s*//")
    # 检查是否为空白行或注释行
    return re.match(comment_reg, line) or re.match(REG_BLANK_LINE, line)


def mapping_input_langs_to_supported(langs: Tuple[str]) -> Set[str]:
    return set(lang for lang in langs if lang in REGS_FOR_LANGS)


def extract_code_from_directory(dirs, langs, output_file):
    support_langs = mapping_input_langs_to_supported(langs)
    print(f"support langs: {support_langs}")
    with open(output_file, "w", encoding="utf-8") as outfile:
        for directory in dirs:
            for root, dirs, files in os.walk(directory):
                for file in files:
                    # get file extend name
                    _, ext = os.path.splitext(file)
                    lang = get_supported_lang(ext)
                    if lang not in REGS_FOR_LANGS or lang not in support_langs:
                        continue
                    print(f"extract code from {root} {file}")
                    file_path = os.path.join(root, file)
                    with open(file_path, "r", encoding="utf-8") as reader:
                        lines = reader.readlines()
                        filtered_lines = [
                            line
                            for line in lines
                            if not is_comment_or_blank(lang, line)
                        ]
                        if filtered_lines:
                            outfile.writelines(filtered_lines)
                            if not filtered_lines[-1].endswith("\n"):
                                outfile.write("\n")
